from .base import *  # noqa: F401, F403

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True  # noqa: F811

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "django-insecure-*5ol0*m=n%@q6&yzmd*e^ftp+j+nv@w!-f9f_fgp93j)xez)0z"

# SECURITY WARNING: define the correct hosts in production!
ALLOWED_HOSTS = ["*"]

EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"



import uuid
from collections import namedtuple

from django import forms
from wagtail import blocks


FieldSpec = namedtuple('FieldSpec', ['name', 'field', 'block'])


class FieldBlockBase(blocks.StructBlock):
    is_field_block = True

    name = blocks.CharBlock()
    label = blocks.CharBlock()
    is_required = blocks.BooleanBlock(required=False, default=True)

    def __init__(self, *args, **kwargs):
        self.field_kwargs = kwargs.pop('field_kwargs', {})
        super().__init__(*args, **kwargs)

    def get_field_spec(self, value):
        kwargs = self.field_kwargs.copy()
        kwargs['required'] = value['is_required']

        return FieldSpec(
            name=value['name'],
            field=self.get_field(value, kwargs),
            block=self
        )

    def clean_field(self, value):
        return value

    def get_field(self, value, field_kwargs):
        return forms.Field(label=value['label'], required=value['is_required'], **field_kwargs)

    def get_context(self, value, parent_context=None):
        context = super().get_context(value, parent_context=parent_context)
        form = context['form']

        context.update({
            'errors': context['form'].errors.get(value['name']),
            'field_id': uuid.uuid4().hex,
            'field_value': form.data.get(value['name'], '')
        })
        return context

    class Meta:
        icon = 'edit'
        group = 'fields'


class TextFieldBlock(FieldBlockBase):
    def get_field(self, value, field_kwargs):
        return forms.CharField(label=value['label'], **field_kwargs)

    class Meta:
        template = 'formation/blocks/text_field.html'
        icon = 'openquote'


class TextareaFieldBlock(FieldBlockBase):
    def get_field(self, value, field_kwargs):
        return forms.CharField(label=value['label'], **field_kwargs)

    class Meta:
        template = 'formation/blocks/textarea_field.html'
        icon = 'openquote'


class IntegerFieldBlock(FieldBlockBase):
    def get_field(self, value, field_kwargs):
        return forms.IntegerField(label=value['label'], **field_kwargs)

    class Meta:
        template = 'formation/blocks/integer_field.html'
        icon = 'order'


class BooleanFieldBlock(FieldBlockBase):
    def get_field(self, value, field_kwargs):
        return forms.BooleanField(label=value['label'], **field_kwargs)

    class Meta:
        template = 'formation/blocks/boolean_field.html'
        icon = 'tick'


class ChoiceFieldBase(FieldBlockBase):
    options = blocks.ListBlock(
        blocks.StructBlock([
            ('name', blocks.CharBlock()),
            ('label', blocks.CharBlock()),
        ])
    )

    def get_field(self, value, field_kwargs):
        return forms.ChoiceField(
            label=value['label'], choices=[
                (option['name'], option['label'])
                for option in value['options']
            ], **field_kwargs
        )


class SubmitButtonBlock(blocks.StructBlock):
    button_text = blocks.CharBlock()

    class Meta:
        template = 'formation/blocks/submit_button.html'
        icon = 'resubmit'
        group = 'fields'


class SelectFieldBlock(ChoiceFieldBase):
    class Meta:
        template = 'formation/blocks/select_field.html'
        icon = 'tasks'


class RadioFieldBlock(ChoiceFieldBase):
    class Meta:
        template = 'formation/blocks/radio_field.html'
        icon = 'radio-empty'


class DateFieldBlock(FieldBlockBase):
    def get_field(self, value, field_kwargs):
        return forms.DateField(label=value['label'], **field_kwargs)

    class Meta:
        template = 'formation/blocks/date_field.html'
        icon = 'date'


class DateTimeFieldBlock(FieldBlockBase):
    def get_field(self, value, field_kwargs):
        return forms.DateTimeField(label=value['label'], **field_kwargs)

    class Meta:
        template = 'formation/blocks/datetime_field.html'
        icon = 'time'


class EmailFieldBlock(FieldBlockBase):
    def get_field(self, value, field_kwargs):
        return forms.EmailField(label=value['label'], **field_kwargs)

    class Meta:
        template = 'formation/blocks/email_field.html'
        icon = 'mail'


class FileFieldBlock(FieldBlockBase):
    def get_context(self, value, parent_context=None):
        context = super().get_context(value, parent_context=parent_context)
        form = context['form']

        context.update({
            'field_value': form.files.get(value['name'], '')
        })
        return context

    def get_field(self, value, field_kwargs):
        return forms.FileField(label=value['label'], **field_kwargs)

    class Meta:
        template = 'formation/blocks/file_field.html'
        icon = 'doc-full'


BASIC_FIELD_SPEC = [
    ('text', TextFieldBlock()),
    ('textarea', TextareaFieldBlock()),
    ('integer', IntegerFieldBlock()),
    ('select', SelectFieldBlock()),
    ('radio', RadioFieldBlock()),
]

FULL_FIELD_SPEC = BASIC_FIELD_SPEC + [
    ('boolean', BooleanFieldBlock()),
    ('date', DateFieldBlock()),
    ('datetime', DateTimeFieldBlock()),
    ('email', EmailFieldBlock()),
]



# TODO: investigate if it makes sense to just generate random key and iv
# this would invalidate a token open restart
from base64 import b64encode, b64decode
import os

from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from django.contrib.contenttypes.models import ContentType
from wagtail.blocks import ListBlock, StreamBlock, StructBlock
from wagtail.fields import StreamField


class TokenProcessor:
    def __init__(self):
        key = os.urandom(32)
        self.iv = os.urandom(16)
        self.cipher = Cipher(algorithms.AES(key), modes.CBC(self.iv))

    def generate_token(self, content_type_id, object_id, block_id):
        content_ids = f'{content_type_id}:{object_id}'.ljust(11)
        to_encrypt = bytes(f'{content_ids}:{block_id}', 'utf-8')

        encryptor = self.cipher.encryptor()
        return str(
            b64encode(encryptor.update(to_encrypt) + encryptor.finalize()),
            'utf-8'
        )

    def unpack_token(self, token_encoded):
        token = b64decode(token_encoded)
        decryptor = self.cipher.decryptor()
        decrypted = decryptor.update(token) + decryptor.finalize()
        content_type_id, object_id, block_id = str(decrypted, 'utf-8').split(':')

        return int(content_type_id), int(object_id), block_id


token_processor = TokenProcessor()


def extract_elements_recursive(container, results, check_method, id_based=False):
    for i, element in enumerate(container):
        if hasattr(container, 'raw_data'):
            block_id = container.raw_data[i]['id']
        elif hasattr(container, 'bound_blocks'):
            block_id = container.bound_blocks[i].id

        if check_method(element):
            element = getattr(element, 'value', element)
            if id_based:
                results[block_id] = element
            else:
                results.append(element)
        elif isinstance(element.block, ListBlock) or isinstance(element.block, StreamBlock):
            extract_elements_recursive(element.value, results, check_method, id_based)
        elif isinstance(element.block, StructBlock):
            value = getattr(element, 'value', element)
            extract_elements_recursive([
                element for _, element in value.bound_blocks.items()
            ], results, check_method, id_based)


def find_block_value(content_type_id, object_id, block_id):
    obj = ContentType.objects.get(pk=content_type_id).model_class().objects.get(pk=object_id)

    fields = [
        getattr(obj, field.name)
        for field in obj._meta.fields
        if isinstance(field, StreamField)
    ]
    form_blocks = {}

    def _formblock_check(element):
        from formation.blocks import BaseFormBlock
        return isinstance(element.block, BaseFormBlock)

    for field in fields:
        extract_elements_recursive(field, form_blocks, _formblock_check, True)

    return form_blocks[block_id]



from wagtail import hooks
from wagtail.snippets.models import register_snippet
from wagtail.snippets.views.snippets import SnippetViewSet

from .models import ReusableForm
from .views import ReusableFormChooserViewSet


class ReusableFormAdmin(SnippetViewSet):
    model = ReusableForm
    icon = "form"
    add_to_admin_menu = True
    list_display = ["name"]


@hooks.register('register_admin_viewset')
def register_reusable_form_chooser_viewset():
    return ReusableFormChooserViewSet('form_chooser', url_prefix='form-chooser')


register_snippet(ReusableFormAdmin)



from wagtail.admin.panels import FieldPanel
from wagtail.fields import StreamField
from wagtail.models import Page

from .blocks import MessageFormBlock


class HomePage(Page):
    content = StreamField([
        ('message_form', MessageFormBlock()),
    ], use_json_field=False)

    content_panels = Page.content_panels + [
        FieldPanel('content')
    ]



# Generated by Django 5.1.8 on 2025-04-13 11:31

import wagtail.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ReusableForm",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=200)),
                ("form_content", wagtail.fields.StreamField([])),
            ],
        ),
    ]



"""
Django settings for testproject project.

Generated by 'django-admin startproject' using Django 4.1.7.

For more information on this file, see
https://docs.djangoproject.com/en/4.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/4.1/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os

PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
BASE_DIR = os.path.dirname(PROJECT_DIR)


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/4.1/howto/deployment/checklist/


# Application definition

INSTALLED_APPS = [
    "home",
    "search",
    "wagtail.contrib.forms",
    "wagtail.contrib.redirects",
    "wagtail.embeds",
    "wagtail.sites",
    "wagtail.users",
    "wagtail.snippets",
    "wagtail.documents",
    "wagtail.images",
    "wagtail.search",
    "wagtail.admin",
    "wagtail",
    "modelcluster",
    "taggit",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",

    "generic_chooser",
    "formation",
]

MIDDLEWARE = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.middleware.security.SecurityMiddleware",
    "wagtail.contrib.redirects.middleware.RedirectMiddleware",
]

ROOT_URLCONF = "testproject.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            os.path.join(PROJECT_DIR, "templates"),
        ],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "testproject.wsgi.application"


# Database
# https://docs.djangoproject.com/en/4.1/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}


# Password validation
# https://docs.djangoproject.com/en/4.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.1/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.1/howto/static-files/

STATICFILES_FINDERS = [
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
]

STATICFILES_DIRS = [
    os.path.join(PROJECT_DIR, "static"),
]

# ManifestStaticFilesStorage is recommended in production, to prevent outdated
# JavaScript / CSS assets being served from cache (e.g. after a Wagtail upgrade).
# See https://docs.djangoproject.com/en/4.1/ref/contrib/staticfiles/#manifeststaticfilesstorage
STATICFILES_STORAGE = "django.contrib.staticfiles.storage.StaticFilesStorage"

STATIC_ROOT = os.path.join(BASE_DIR, "static")
STATIC_URL = "/static/"

MEDIA_ROOT = os.path.join(BASE_DIR, "media")
MEDIA_URL = "/media/"


# Wagtail settings

WAGTAIL_SITE_NAME = "testproject"

# Search
# https://docs.wagtail.org/en/stable/topics/search/backends.html
WAGTAILSEARCH_BACKENDS = {
    "default": {
        "BACKEND": "wagtail.search.backends.database",
    }
}

# Base URL to use when referring to full URLs within the Wagtail admin backend -
# e.g. in notification emails. Don't include '/admin' or a trailing slash
WAGTAILADMIN_BASE_URL = "http://example.com"

DEBUG = True
MIGRATION_MODULES = {
    'formation': 'formation_migrations',
}



# -*- coding: utf-8 -*-
from django.db import migrations


def create_homepage(apps, schema_editor):
    # Get models
    ContentType = apps.get_model("contenttypes.ContentType")
    Page = apps.get_model("wagtailcore.Page")
    Site = apps.get_model("wagtailcore.Site")
    HomePage = apps.get_model("home.HomePage")

    # Delete the default homepage
    # If migration is run multiple times, it may have already been deleted
    Page.objects.filter(id=2).delete()

    # Create content type for homepage model
    homepage_content_type, __ = ContentType.objects.get_or_create(
        model="homepage", app_label="home"
    )

    # Create a new homepage
    homepage = HomePage.objects.create(
        title="Home",
        draft_title="Home",
        slug="home",
        content_type=homepage_content_type,
        path="00010001",
        depth=2,
        numchild=0,
        url_path="/home/",
    )

    # Create a site with the new homepage set as the root
    Site.objects.create(hostname="localhost", root_page=homepage, is_default_site=True)


def remove_homepage(apps, schema_editor):
    # Get models
    ContentType = apps.get_model("contenttypes.ContentType")
    HomePage = apps.get_model("home.HomePage")

    # Delete the default homepage
    # Page and Site objects CASCADE
    HomePage.objects.filter(slug="home", depth=2).delete()

    # Delete content type for homepage model
    ContentType.objects.filter(model="homepage", app_label="home").delete()


class Migration(migrations.Migration):

    run_before = [
        ("wagtailcore", "0053_locale_model"),
    ]

    dependencies = [
        ("home", "0001_initial"),
    ]

    operations = [
        migrations.RunPython(create_homepage, remove_homepage),
    ]
